# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from .doctag import (
    DoctagResource,
    AsyncDoctagResource,
    DoctagResourceWithRawResponse,
    AsyncDoctagResourceWithRawResponse,
    DoctagResourceWithStreamingResponse,
    AsyncDoctagResourceWithStreamingResponse,
)
from ...._types import (
    Body,
    Omit,
    Query,
    Headers,
    NoneType,
    NotGiven,
    FileTypes,
    SequenceNotStr,
    omit,
    not_given,
)
from ...._utils import (
    extract_files,
    maybe_transform,
    strip_not_given,
    deepcopy_minimal,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.api import (
    document_delete_params,
    document_update_params,
    document_upload_params,
    document_retrieve_params,
    document_upload_from_url_params,
)
from ...._base_client import make_request_options
from ....types.api.document_update_response import DocumentUpdateResponse
from ....types.api.document_upload_response import DocumentUploadResponse
from ....types.api.document_retrieve_response import DocumentRetrieveResponse
from ....types.api.document_get_parsed_response import DocumentGetParsedResponse
from ....types.api.document_upload_from_url_response import DocumentUploadFromURLResponse

__all__ = ["DocumentResource", "AsyncDocumentResource"]


class DocumentResource(SyncAPIResource):
    @cached_property
    def doctag(self) -> DoctagResource:
        return DoctagResource(self._client)

    @cached_property
    def with_raw_response(self) -> DocumentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#accessing-raw-response-data-eg-headers
        """
        return DocumentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DocumentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#with_streaming_response
        """
        return DocumentResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        external_ids: SequenceNotStr[str],
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentRetrieveResponse:
        """
        Get one or more documents by external IDs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return self._get(
            "/api/document/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"external_ids": external_ids}, document_retrieve_params.DocumentRetrieveParams),
            ),
            cast_to=DocumentRetrieveResponse,
        )

    def update(
        self,
        *,
        documents: Iterable[document_update_params.Document],
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUpdateResponse:
        """
        Update one or more documents.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return self._patch(
            "/api/document/",
            body=maybe_transform({"documents": documents}, document_update_params.DocumentUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUpdateResponse,
        )

    def delete(
        self,
        *,
        external_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete one or more documents.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            "/api/document/",
            body=maybe_transform({"external_ids": external_ids}, document_delete_params.DocumentDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def download(
        self,
        document_ext_id: str,
        *,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Download a document by its external ID.

        Retrieves and decrypts the document for
        downloading as an attachment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_ext_id:
            raise ValueError(f"Expected a non-empty value for `document_ext_id` but received {document_ext_id!r}")
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return self._get(
            f"/api/document/{document_ext_id}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get_parsed(
        self,
        stage: Literal["marker", "subchunk", "final"],
        *,
        document_ext_id: str,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentGetParsedResponse:
        """Retrieve the full parsed document to be handled by the frontend.

        Only requires
        document_ext_id, workspace is determined through RLS.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_ext_id:
            raise ValueError(f"Expected a non-empty value for `document_ext_id` but received {document_ext_id!r}")
        if not stage:
            raise ValueError(f"Expected a non-empty value for `stage` but received {stage!r}")
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return self._get(
            f"/api/document/{document_ext_id}/parsed-{stage}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentGetParsedResponse,
        )

    def upload(
        self,
        *,
        workspace_ext_id: str,
        files: SequenceNotStr[FileTypes],
        config_ext_id: Optional[str] | Omit = omit,
        shared: bool | Omit = omit,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadResponse:
        """Upload multiple documents to a workspace with encryption.

        Documents are queued
        for processing, parsed, and indexed for vector search.

        Requires active subscription (paid/trial/dev) if Stripe is configured.

        Args:
          files: Multiple files to upload

          config_ext_id: Configuration to use for processing

          shared: Whether the document should be shared with workspace members

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        body = deepcopy_minimal({"files": files})
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/api/document/upload",
            body=maybe_transform(body, document_upload_params.DocumentUploadParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "workspace_ext_id": workspace_ext_id,
                        "config_ext_id": config_ext_id,
                        "shared": shared,
                    },
                    document_upload_params.DocumentUploadParams,
                ),
            ),
            cast_to=DocumentUploadResponse,
        )

    def upload_from_url(
        self,
        *,
        urls: SequenceNotStr[str],
        workspace_ext_id: str,
        config_ext_id: Optional[str] | Omit = omit,
        shared: bool | Omit = omit,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadFromURLResponse:
        """
        Download and upload documents from URLs to a workspace with encryption.
        Documents are queued for processing, parsed, and indexed for vector search.

        Requires active subscription (paid/trial/dev) if Stripe is configured.

        Args:
          urls: URLs to download documents from

          config_ext_id: Configuration to use for processing

          shared: Whether the document should be shared with workspace members

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return self._post(
            "/api/document/upload-url",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "urls": urls,
                        "workspace_ext_id": workspace_ext_id,
                        "config_ext_id": config_ext_id,
                        "shared": shared,
                    },
                    document_upload_from_url_params.DocumentUploadFromURLParams,
                ),
            ),
            cast_to=DocumentUploadFromURLResponse,
        )

    def view(
        self,
        document_ext_id: str,
        *,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """View a document inline in the browser.

        Retrieves and decrypts the document for
        inline viewing with optional page specification.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_ext_id:
            raise ValueError(f"Expected a non-empty value for `document_ext_id` but received {document_ext_id!r}")
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return self._get(
            f"/api/document/{document_ext_id}/view",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncDocumentResource(AsyncAPIResource):
    @cached_property
    def doctag(self) -> AsyncDoctagResource:
        return AsyncDoctagResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDocumentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDocumentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDocumentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#with_streaming_response
        """
        return AsyncDocumentResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        external_ids: SequenceNotStr[str],
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentRetrieveResponse:
        """
        Get one or more documents by external IDs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return await self._get(
            "/api/document/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"external_ids": external_ids}, document_retrieve_params.DocumentRetrieveParams
                ),
            ),
            cast_to=DocumentRetrieveResponse,
        )

    async def update(
        self,
        *,
        documents: Iterable[document_update_params.Document],
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUpdateResponse:
        """
        Update one or more documents.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return await self._patch(
            "/api/document/",
            body=await async_maybe_transform({"documents": documents}, document_update_params.DocumentUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUpdateResponse,
        )

    async def delete(
        self,
        *,
        external_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete one or more documents.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            "/api/document/",
            body=await async_maybe_transform(
                {"external_ids": external_ids}, document_delete_params.DocumentDeleteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def download(
        self,
        document_ext_id: str,
        *,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Download a document by its external ID.

        Retrieves and decrypts the document for
        downloading as an attachment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_ext_id:
            raise ValueError(f"Expected a non-empty value for `document_ext_id` but received {document_ext_id!r}")
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return await self._get(
            f"/api/document/{document_ext_id}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get_parsed(
        self,
        stage: Literal["marker", "subchunk", "final"],
        *,
        document_ext_id: str,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentGetParsedResponse:
        """Retrieve the full parsed document to be handled by the frontend.

        Only requires
        document_ext_id, workspace is determined through RLS.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_ext_id:
            raise ValueError(f"Expected a non-empty value for `document_ext_id` but received {document_ext_id!r}")
        if not stage:
            raise ValueError(f"Expected a non-empty value for `stage` but received {stage!r}")
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return await self._get(
            f"/api/document/{document_ext_id}/parsed-{stage}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentGetParsedResponse,
        )

    async def upload(
        self,
        *,
        workspace_ext_id: str,
        files: SequenceNotStr[FileTypes],
        config_ext_id: Optional[str] | Omit = omit,
        shared: bool | Omit = omit,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadResponse:
        """Upload multiple documents to a workspace with encryption.

        Documents are queued
        for processing, parsed, and indexed for vector search.

        Requires active subscription (paid/trial/dev) if Stripe is configured.

        Args:
          files: Multiple files to upload

          config_ext_id: Configuration to use for processing

          shared: Whether the document should be shared with workspace members

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        body = deepcopy_minimal({"files": files})
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/api/document/upload",
            body=await async_maybe_transform(body, document_upload_params.DocumentUploadParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "workspace_ext_id": workspace_ext_id,
                        "config_ext_id": config_ext_id,
                        "shared": shared,
                    },
                    document_upload_params.DocumentUploadParams,
                ),
            ),
            cast_to=DocumentUploadResponse,
        )

    async def upload_from_url(
        self,
        *,
        urls: SequenceNotStr[str],
        workspace_ext_id: str,
        config_ext_id: Optional[str] | Omit = omit,
        shared: bool | Omit = omit,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadFromURLResponse:
        """
        Download and upload documents from URLs to a workspace with encryption.
        Documents are queued for processing, parsed, and indexed for vector search.

        Requires active subscription (paid/trial/dev) if Stripe is configured.

        Args:
          urls: URLs to download documents from

          config_ext_id: Configuration to use for processing

          shared: Whether the document should be shared with workspace members

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return await self._post(
            "/api/document/upload-url",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "urls": urls,
                        "workspace_ext_id": workspace_ext_id,
                        "config_ext_id": config_ext_id,
                        "shared": shared,
                    },
                    document_upload_from_url_params.DocumentUploadFromURLParams,
                ),
            ),
            cast_to=DocumentUploadFromURLResponse,
        )

    async def view(
        self,
        document_ext_id: str,
        *,
        workspace_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """View a document inline in the browser.

        Retrieves and decrypts the document for
        inline viewing with optional page specification.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_ext_id:
            raise ValueError(f"Expected a non-empty value for `document_ext_id` but received {document_ext_id!r}")
        extra_headers = {**strip_not_given({"workspace-key": workspace_key}), **(extra_headers or {})}
        return await self._get(
            f"/api/document/{document_ext_id}/view",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class DocumentResourceWithRawResponse:
    def __init__(self, document: DocumentResource) -> None:
        self._document = document

        self.retrieve = to_raw_response_wrapper(
            document.retrieve,
        )
        self.update = to_raw_response_wrapper(
            document.update,
        )
        self.delete = to_raw_response_wrapper(
            document.delete,
        )
        self.download = to_raw_response_wrapper(
            document.download,
        )
        self.get_parsed = to_raw_response_wrapper(
            document.get_parsed,
        )
        self.upload = to_raw_response_wrapper(
            document.upload,
        )
        self.upload_from_url = to_raw_response_wrapper(
            document.upload_from_url,
        )
        self.view = to_raw_response_wrapper(
            document.view,
        )

    @cached_property
    def doctag(self) -> DoctagResourceWithRawResponse:
        return DoctagResourceWithRawResponse(self._document.doctag)


class AsyncDocumentResourceWithRawResponse:
    def __init__(self, document: AsyncDocumentResource) -> None:
        self._document = document

        self.retrieve = async_to_raw_response_wrapper(
            document.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            document.update,
        )
        self.delete = async_to_raw_response_wrapper(
            document.delete,
        )
        self.download = async_to_raw_response_wrapper(
            document.download,
        )
        self.get_parsed = async_to_raw_response_wrapper(
            document.get_parsed,
        )
        self.upload = async_to_raw_response_wrapper(
            document.upload,
        )
        self.upload_from_url = async_to_raw_response_wrapper(
            document.upload_from_url,
        )
        self.view = async_to_raw_response_wrapper(
            document.view,
        )

    @cached_property
    def doctag(self) -> AsyncDoctagResourceWithRawResponse:
        return AsyncDoctagResourceWithRawResponse(self._document.doctag)


class DocumentResourceWithStreamingResponse:
    def __init__(self, document: DocumentResource) -> None:
        self._document = document

        self.retrieve = to_streamed_response_wrapper(
            document.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            document.update,
        )
        self.delete = to_streamed_response_wrapper(
            document.delete,
        )
        self.download = to_streamed_response_wrapper(
            document.download,
        )
        self.get_parsed = to_streamed_response_wrapper(
            document.get_parsed,
        )
        self.upload = to_streamed_response_wrapper(
            document.upload,
        )
        self.upload_from_url = to_streamed_response_wrapper(
            document.upload_from_url,
        )
        self.view = to_streamed_response_wrapper(
            document.view,
        )

    @cached_property
    def doctag(self) -> DoctagResourceWithStreamingResponse:
        return DoctagResourceWithStreamingResponse(self._document.doctag)


class AsyncDocumentResourceWithStreamingResponse:
    def __init__(self, document: AsyncDocumentResource) -> None:
        self._document = document

        self.retrieve = async_to_streamed_response_wrapper(
            document.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            document.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            document.delete,
        )
        self.download = async_to_streamed_response_wrapper(
            document.download,
        )
        self.get_parsed = async_to_streamed_response_wrapper(
            document.get_parsed,
        )
        self.upload = async_to_streamed_response_wrapper(
            document.upload,
        )
        self.upload_from_url = async_to_streamed_response_wrapper(
            document.upload_from_url,
        )
        self.view = async_to_streamed_response_wrapper(
            document.view,
        )

    @cached_property
    def doctag(self) -> AsyncDoctagResourceWithStreamingResponse:
        return AsyncDoctagResourceWithStreamingResponse(self._document.doctag)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from .contacts import (
    ContactsResource,
    AsyncContactsResource,
    ContactsResourceWithRawResponse,
    AsyncContactsResourceWithRawResponse,
    ContactsResourceWithStreamingResponse,
    AsyncContactsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.api import (
    user_login_params,
    user_register_params,
    user_verify_email_params,
    user_change_password_params,
    user_check_sso_status_params,
)
from .subscription import (
    SubscriptionResource,
    AsyncSubscriptionResource,
    SubscriptionResourceWithRawResponse,
    AsyncSubscriptionResourceWithRawResponse,
    SubscriptionResourceWithStreamingResponse,
    AsyncSubscriptionResourceWithStreamingResponse,
)
from ...._base_client import make_request_options
from ....types.api.user_login_response import UserLoginResponse
from ....types.api.user_logout_response import UserLogoutResponse
from ....types.api.user_verify_email_response import UserVerifyEmailResponse
from ....types.api.user_list_products_response import UserListProductsResponse
from ....types.api.user_change_password_response import UserChangePasswordResponse
from ....types.api.user_list_workspaces_response import UserListWorkspacesResponse
from ....types.api.user_check_sso_status_response import UserCheckSSOStatusResponse

__all__ = ["UserResource", "AsyncUserResource"]


class UserResource(SyncAPIResource):
    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def subscription(self) -> SubscriptionResource:
        return SubscriptionResource(self._client)

    @cached_property
    def contacts(self) -> ContactsResource:
        return ContactsResource(self._client)

    @cached_property
    def with_raw_response(self) -> UserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#accessing-raw-response-data-eg-headers
        """
        return UserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#with_streaming_response
        """
        return UserResourceWithStreamingResponse(self)

    def change_password(
        self,
        *,
        new_signing_key: str,
        rewrapped_workspace_keys: Dict[str, str],
        signature: str,
        timestamp: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserChangePasswordResponse:
        """Change user's master password by re-keying all workspace keys.

        Client must:

        1.

        Sign "email|timestamp" with current Ed25519 key (proves current password)
        2. Provide new Ed25519 signing key (derived from new password)
        3. Re-wrap all workspace keys with new X25519 public key

        Server will:

        1. Verify signature with stored signing_key_pub
        2. Derive new X25519 encryption key from new Ed25519 signing key
        3. Update both keys and all workspace wrapped keys

        Note: This changes the master password (encryption password), not authentication
        password. Both local and SSO users can change their master password.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/user/change_password",
            body=maybe_transform(
                {
                    "new_signing_key": new_signing_key,
                    "rewrapped_workspace_keys": rewrapped_workspace_keys,
                    "signature": signature,
                    "timestamp": timestamp,
                },
                user_change_password_params.UserChangePasswordParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserChangePasswordResponse,
        )

    def check_sso_status(
        self,
        *,
        email: str,
        sso_token: str,
        family_name: Optional[str] | Omit = omit,
        given_name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserCheckSSOStatusResponse:
        """Check SSO user registration state.

        Returns one of three states:

        1.

        new_user: No user exists with this email
        2. local_exists: User exists but registered locally (can link to SSO)
        3. sso_exists: User exists and already linked to SSO

        Frontend uses this to show appropriate UI:

        - new_user -> "Set Master Password"
        - local_exists -> "Link SSO? Enter current master password"
        - sso_exists -> "Enter Master Password"

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/user/sso-status",
            body=maybe_transform(
                {
                    "email": email,
                    "sso_token": sso_token,
                    "family_name": family_name,
                    "given_name": given_name,
                    "picture": picture,
                },
                user_check_sso_status_params.UserCheckSSOStatusParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserCheckSSOStatusResponse,
        )

    def list_products(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListProductsResponse:
        """Get available subscription products and prices from Stripe."""
        return self._get(
            "/api/user/products",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserListProductsResponse,
        )

    def list_workspaces(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListWorkspacesResponse:
        """
        Retrieve the list of workspaces associated with the current authenticated user.
        Includes wrapped_key, stats (conversation/document counts with shared/private
        breakdown), and users for each workspace. All data is fetched efficiently using
        batch queries to avoid N+1 problems. Leverages RLS to enforce access control.
        """
        return self._get(
            "/api/user/workspaces",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserListWorkspacesResponse,
        )

    def login(
        self,
        *,
        email: str,
        signature: str,
        timestamp: int,
        sso_token: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserLoginResponse:
        """
        Login with Ed25519 signature verification (local or SSO).

        Authentication flow:

        1. Client derives Ed25519 keypair from password
        2. Client signs "email|timestamp" with Ed25519 private key
        3. Server verifies signature using stored Ed25519 public key
        4. Server encrypts response with stored X25519 public key

        For SSO users: Also validates SSO token before proceeding.

        Returns encrypted login response that only the correct password can decrypt.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/user/login",
            body=maybe_transform(
                {
                    "email": email,
                    "signature": signature,
                    "timestamp": timestamp,
                    "sso_token": sso_token,
                },
                user_login_params.UserLoginParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserLoginResponse,
        )

    def logout(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserLogoutResponse:
        """Log out the current user by clearing the refresh token cookie and session key."""
        return self._post(
            "/api/user/logout",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserLogoutResponse,
        )

    def register(
        self,
        *,
        email: str,
        signing_key: str,
        verification_credential: str,
        family_name: Optional[str] | Omit = omit,
        given_name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Register a new user with email verification (local or SSO).

        Accepts either:

        - 3-word verification code for local users
        - SSO JWT token for SSO users

        Auto-detects credential type and handles both flows.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/user/register",
            body=maybe_transform(
                {
                    "email": email,
                    "signing_key": signing_key,
                    "verification_credential": verification_credential,
                    "family_name": family_name,
                    "given_name": given_name,
                    "picture": picture,
                },
                user_register_params.UserRegisterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def verify_email(
        self,
        *,
        email: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserVerifyEmailResponse:
        """Send verification email with 3-word code to user.

        Calls central server to send
        the email.

        Note: Fails silently if email already exists to prevent email enumeration
        attacks. Also returns success even on rate limit/errors to avoid information
        disclosure.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/user/verify-email",
            body=maybe_transform({"email": email}, user_verify_email_params.UserVerifyEmailParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserVerifyEmailResponse,
        )


class AsyncUserResource(AsyncAPIResource):
    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def subscription(self) -> AsyncSubscriptionResource:
        return AsyncSubscriptionResource(self._client)

    @cached_property
    def contacts(self) -> AsyncContactsResource:
        return AsyncContactsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#with_streaming_response
        """
        return AsyncUserResourceWithStreamingResponse(self)

    async def change_password(
        self,
        *,
        new_signing_key: str,
        rewrapped_workspace_keys: Dict[str, str],
        signature: str,
        timestamp: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserChangePasswordResponse:
        """Change user's master password by re-keying all workspace keys.

        Client must:

        1.

        Sign "email|timestamp" with current Ed25519 key (proves current password)
        2. Provide new Ed25519 signing key (derived from new password)
        3. Re-wrap all workspace keys with new X25519 public key

        Server will:

        1. Verify signature with stored signing_key_pub
        2. Derive new X25519 encryption key from new Ed25519 signing key
        3. Update both keys and all workspace wrapped keys

        Note: This changes the master password (encryption password), not authentication
        password. Both local and SSO users can change their master password.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/user/change_password",
            body=await async_maybe_transform(
                {
                    "new_signing_key": new_signing_key,
                    "rewrapped_workspace_keys": rewrapped_workspace_keys,
                    "signature": signature,
                    "timestamp": timestamp,
                },
                user_change_password_params.UserChangePasswordParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserChangePasswordResponse,
        )

    async def check_sso_status(
        self,
        *,
        email: str,
        sso_token: str,
        family_name: Optional[str] | Omit = omit,
        given_name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserCheckSSOStatusResponse:
        """Check SSO user registration state.

        Returns one of three states:

        1.

        new_user: No user exists with this email
        2. local_exists: User exists but registered locally (can link to SSO)
        3. sso_exists: User exists and already linked to SSO

        Frontend uses this to show appropriate UI:

        - new_user -> "Set Master Password"
        - local_exists -> "Link SSO? Enter current master password"
        - sso_exists -> "Enter Master Password"

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/user/sso-status",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "sso_token": sso_token,
                    "family_name": family_name,
                    "given_name": given_name,
                    "picture": picture,
                },
                user_check_sso_status_params.UserCheckSSOStatusParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserCheckSSOStatusResponse,
        )

    async def list_products(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListProductsResponse:
        """Get available subscription products and prices from Stripe."""
        return await self._get(
            "/api/user/products",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserListProductsResponse,
        )

    async def list_workspaces(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListWorkspacesResponse:
        """
        Retrieve the list of workspaces associated with the current authenticated user.
        Includes wrapped_key, stats (conversation/document counts with shared/private
        breakdown), and users for each workspace. All data is fetched efficiently using
        batch queries to avoid N+1 problems. Leverages RLS to enforce access control.
        """
        return await self._get(
            "/api/user/workspaces",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserListWorkspacesResponse,
        )

    async def login(
        self,
        *,
        email: str,
        signature: str,
        timestamp: int,
        sso_token: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserLoginResponse:
        """
        Login with Ed25519 signature verification (local or SSO).

        Authentication flow:

        1. Client derives Ed25519 keypair from password
        2. Client signs "email|timestamp" with Ed25519 private key
        3. Server verifies signature using stored Ed25519 public key
        4. Server encrypts response with stored X25519 public key

        For SSO users: Also validates SSO token before proceeding.

        Returns encrypted login response that only the correct password can decrypt.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/user/login",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "signature": signature,
                    "timestamp": timestamp,
                    "sso_token": sso_token,
                },
                user_login_params.UserLoginParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserLoginResponse,
        )

    async def logout(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserLogoutResponse:
        """Log out the current user by clearing the refresh token cookie and session key."""
        return await self._post(
            "/api/user/logout",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserLogoutResponse,
        )

    async def register(
        self,
        *,
        email: str,
        signing_key: str,
        verification_credential: str,
        family_name: Optional[str] | Omit = omit,
        given_name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Register a new user with email verification (local or SSO).

        Accepts either:

        - 3-word verification code for local users
        - SSO JWT token for SSO users

        Auto-detects credential type and handles both flows.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/user/register",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "signing_key": signing_key,
                    "verification_credential": verification_credential,
                    "family_name": family_name,
                    "given_name": given_name,
                    "picture": picture,
                },
                user_register_params.UserRegisterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def verify_email(
        self,
        *,
        email: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserVerifyEmailResponse:
        """Send verification email with 3-word code to user.

        Calls central server to send
        the email.

        Note: Fails silently if email already exists to prevent email enumeration
        attacks. Also returns success even on rate limit/errors to avoid information
        disclosure.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/user/verify-email",
            body=await async_maybe_transform({"email": email}, user_verify_email_params.UserVerifyEmailParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserVerifyEmailResponse,
        )


class UserResourceWithRawResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.change_password = to_raw_response_wrapper(
            user.change_password,
        )
        self.check_sso_status = to_raw_response_wrapper(
            user.check_sso_status,
        )
        self.list_products = to_raw_response_wrapper(
            user.list_products,
        )
        self.list_workspaces = to_raw_response_wrapper(
            user.list_workspaces,
        )
        self.login = to_raw_response_wrapper(
            user.login,
        )
        self.logout = to_raw_response_wrapper(
            user.logout,
        )
        self.register = to_raw_response_wrapper(
            user.register,
        )
        self.verify_email = to_raw_response_wrapper(
            user.verify_email,
        )

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._user.settings)

    @cached_property
    def subscription(self) -> SubscriptionResourceWithRawResponse:
        return SubscriptionResourceWithRawResponse(self._user.subscription)

    @cached_property
    def contacts(self) -> ContactsResourceWithRawResponse:
        return ContactsResourceWithRawResponse(self._user.contacts)


class AsyncUserResourceWithRawResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.change_password = async_to_raw_response_wrapper(
            user.change_password,
        )
        self.check_sso_status = async_to_raw_response_wrapper(
            user.check_sso_status,
        )
        self.list_products = async_to_raw_response_wrapper(
            user.list_products,
        )
        self.list_workspaces = async_to_raw_response_wrapper(
            user.list_workspaces,
        )
        self.login = async_to_raw_response_wrapper(
            user.login,
        )
        self.logout = async_to_raw_response_wrapper(
            user.logout,
        )
        self.register = async_to_raw_response_wrapper(
            user.register,
        )
        self.verify_email = async_to_raw_response_wrapper(
            user.verify_email,
        )

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._user.settings)

    @cached_property
    def subscription(self) -> AsyncSubscriptionResourceWithRawResponse:
        return AsyncSubscriptionResourceWithRawResponse(self._user.subscription)

    @cached_property
    def contacts(self) -> AsyncContactsResourceWithRawResponse:
        return AsyncContactsResourceWithRawResponse(self._user.contacts)


class UserResourceWithStreamingResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.change_password = to_streamed_response_wrapper(
            user.change_password,
        )
        self.check_sso_status = to_streamed_response_wrapper(
            user.check_sso_status,
        )
        self.list_products = to_streamed_response_wrapper(
            user.list_products,
        )
        self.list_workspaces = to_streamed_response_wrapper(
            user.list_workspaces,
        )
        self.login = to_streamed_response_wrapper(
            user.login,
        )
        self.logout = to_streamed_response_wrapper(
            user.logout,
        )
        self.register = to_streamed_response_wrapper(
            user.register,
        )
        self.verify_email = to_streamed_response_wrapper(
            user.verify_email,
        )

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._user.settings)

    @cached_property
    def subscription(self) -> SubscriptionResourceWithStreamingResponse:
        return SubscriptionResourceWithStreamingResponse(self._user.subscription)

    @cached_property
    def contacts(self) -> ContactsResourceWithStreamingResponse:
        return ContactsResourceWithStreamingResponse(self._user.contacts)


class AsyncUserResourceWithStreamingResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.change_password = async_to_streamed_response_wrapper(
            user.change_password,
        )
        self.check_sso_status = async_to_streamed_response_wrapper(
            user.check_sso_status,
        )
        self.list_products = async_to_streamed_response_wrapper(
            user.list_products,
        )
        self.list_workspaces = async_to_streamed_response_wrapper(
            user.list_workspaces,
        )
        self.login = async_to_streamed_response_wrapper(
            user.login,
        )
        self.logout = async_to_streamed_response_wrapper(
            user.logout,
        )
        self.register = async_to_streamed_response_wrapper(
            user.register,
        )
        self.verify_email = async_to_streamed_response_wrapper(
            user.verify_email,
        )

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._user.settings)

    @cached_property
    def subscription(self) -> AsyncSubscriptionResourceWithStreamingResponse:
        return AsyncSubscriptionResourceWithStreamingResponse(self._user.subscription)

    @cached_property
    def contacts(self) -> AsyncContactsResourceWithStreamingResponse:
        return AsyncContactsResourceWithStreamingResponse(self._user.contacts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["WorkspaceGetStatsResponse"]


class WorkspaceGetStatsResponse(BaseModel):
    private_conversation_count: Optional[int] = None

    private_document_count: Optional[int] = None

    shared_conversation_count: Optional[int] = None

    shared_document_count: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["DocumentUploadFromURLResponse"]


class DocumentUploadFromURLResponse(BaseModel):
    """Response for document upload operations.

    Returns array of document IDs in the same order as uploaded files.
    """

    doc_ext_ids: List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UserResponse"]


class UserResponse(BaseModel):
    """Standard user representation used across all endpoints.

    Used for: login response, workspace users, contacts (when registered).
    """

    email: str

    encryption_public_key: str

    external_id: str

    family_name: str

    given_name: str

    picture: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["EmbedderConfig"]


class EmbedderConfig(BaseModel):
    api_type: Optional[Literal["local", "remote"]] = FieldInfo(alias="API_TYPE", default=None)
    """The inference type (local or remote)."""

    batch_size: Optional[int] = FieldInfo(alias="BATCH_SIZE", default=None)
    """Smaller batch size for better parallelization."""

    embed_prefix: Optional[str] = FieldInfo(alias="EMBED_PREFIX", default=None)
    """How to embed the sentence for retrieval."""

    max_concurrent_requests: Optional[int] = FieldInfo(alias="MAX_CONCURRENT_REQUESTS", default=None)
    """Adjust concurrency level as needed."""

    api_model_name: Optional[str] = FieldInfo(alias="MODEL_NAME", default=None)
    """The name of the embedder model."""

    query_prefix: Optional[str] = FieldInfo(alias="QUERY_PREFIX", default=None)
    """How to embed the sentence for query."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["DoctagGenerateParams"]


class DoctagGenerateParams(TypedDict, total=False):
    doc_ext_ids: Required[SequenceNotStr[str]]

    tag_ext_ids: Required[SequenceNotStr[str]]

    config_ext_id: Optional[str]
    """Configuration to use for LLM"""

    workspace_key: Annotated[str, PropertyInfo(alias="workspace-key")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["WorkspaceCreateProtectedParams"]


class WorkspaceCreateProtectedParams(TypedDict, total=False):
    name: Required[str]

    description: Optional[str]

    is_public: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ConfigCreateResponse"]


class ConfigCreateResponse(BaseModel):
    """Response model for configuration save endpoint"""

    created_at: str

    external_id: str

    title: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date, datetime

from ..._models import BaseModel
from .document.doc_tag_response import DocTagResponse

__all__ = ["DocResponse", "DocMetadata"]


class DocMetadata(BaseModel):
    """Structured model for document metadata stored in JSONB column."""

    doc_author: Optional[str] = None

    doc_date: Optional[date] = None

    doc_nature: Optional[str] = None

    doc_subject: Optional[str] = None

    title: Optional[str] = None


class DocResponse(BaseModel):
    created_at: datetime

    created_by_ext_id: str

    external_id: str

    updated_at: datetime

    workspace_ext_id: str

    config_ext_id: Optional[str] = None

    doc_metadata: Optional[DocMetadata] = None
    """Structured model for document metadata stored in JSONB column."""

    doctags: Optional[List[DocTagResponse]] = None

    file_name: Optional[str] = None

    file_size: Optional[int] = None

    file_type: Optional[str] = None

    n_chunks: Optional[int] = None

    n_pages: Optional[int] = None

    re_ocred: Optional[bool] = None

    shared: Optional[bool] = None

    status: Optional[str] = None

    storage_type: Optional[str] = None

    storage_uri: Optional[str] = None

    tokens: Optional[int] = None

    updated_by_ext_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .api import (
    APIResource,
    AsyncAPIResource,
    APIResourceWithRawResponse,
    AsyncAPIResourceWithRawResponse,
    APIResourceWithStreamingResponse,
    AsyncAPIResourceWithStreamingResponse,
)
from .tag import (
    TagResource,
    AsyncTagResource,
    TagResourceWithRawResponse,
    AsyncTagResourceWithRawResponse,
    TagResourceWithStreamingResponse,
    AsyncTagResourceWithStreamingResponse,
)
from .user import (
    UserResource,
    AsyncUserResource,
    UserResourceWithRawResponse,
    AsyncUserResourceWithRawResponse,
    UserResourceWithStreamingResponse,
    AsyncUserResourceWithStreamingResponse,
)
from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)
from .configs import (
    ConfigsResource,
    AsyncConfigsResource,
    ConfigsResourceWithRawResponse,
    AsyncConfigsResourceWithRawResponse,
    ConfigsResourceWithStreamingResponse,
    AsyncConfigsResourceWithStreamingResponse,
)
from .document import (
    DocumentResource,
    AsyncDocumentResource,
    DocumentResourceWithRawResponse,
    AsyncDocumentResourceWithRawResponse,
    DocumentResourceWithStreamingResponse,
    AsyncDocumentResourceWithStreamingResponse,
)
from .assistant import (
    AssistantResource,
    AsyncAssistantResource,
    AssistantResourceWithRawResponse,
    AsyncAssistantResourceWithRawResponse,
    AssistantResourceWithStreamingResponse,
    AsyncAssistantResourceWithStreamingResponse,
)
from .workspace import (
    WorkspaceResource,
    AsyncWorkspaceResource,
    WorkspaceResourceWithRawResponse,
    AsyncWorkspaceResourceWithRawResponse,
    WorkspaceResourceWithStreamingResponse,
    AsyncWorkspaceResourceWithStreamingResponse,
)
from .conversation import (
    ConversationResource,
    AsyncConversationResource,
    ConversationResourceWithRawResponse,
    AsyncConversationResourceWithRawResponse,
    ConversationResourceWithStreamingResponse,
    AsyncConversationResourceWithStreamingResponse,
)
from .notifications import (
    NotificationsResource,
    AsyncNotificationsResource,
    NotificationsResourceWithRawResponse,
    AsyncNotificationsResourceWithRawResponse,
    NotificationsResourceWithStreamingResponse,
    AsyncNotificationsResourceWithStreamingResponse,
)

__all__ = [
    "UserResource",
    "AsyncUserResource",
    "UserResourceWithRawResponse",
    "AsyncUserResourceWithRawResponse",
    "UserResourceWithStreamingResponse",
    "AsyncUserResourceWithStreamingResponse",
    "WorkspaceResource",
    "AsyncWorkspaceResource",
    "WorkspaceResourceWithRawResponse",
    "AsyncWorkspaceResourceWithRawResponse",
    "WorkspaceResourceWithStreamingResponse",
    "AsyncWorkspaceResourceWithStreamingResponse",
    "DocumentResource",
    "AsyncDocumentResource",
    "DocumentResourceWithRawResponse",
    "AsyncDocumentResourceWithRawResponse",
    "DocumentResourceWithStreamingResponse",
    "AsyncDocumentResourceWithStreamingResponse",
    "ConversationResource",
    "AsyncConversationResource",
    "ConversationResourceWithRawResponse",
    "AsyncConversationResourceWithRawResponse",
    "ConversationResourceWithStreamingResponse",
    "AsyncConversationResourceWithStreamingResponse",
    "AssistantResource",
    "AsyncAssistantResource",
    "AssistantResourceWithRawResponse",
    "AsyncAssistantResourceWithRawResponse",
    "AssistantResourceWithStreamingResponse",
    "AsyncAssistantResourceWithStreamingResponse",
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "TagResource",
    "AsyncTagResource",
    "TagResourceWithRawResponse",
    "AsyncTagResourceWithRawResponse",
    "TagResourceWithStreamingResponse",
    "AsyncTagResourceWithStreamingResponse",
    "ConfigsResource",
    "AsyncConfigsResource",
    "ConfigsResourceWithRawResponse",
    "AsyncConfigsResourceWithRawResponse",
    "ConfigsResourceWithStreamingResponse",
    "AsyncConfigsResourceWithStreamingResponse",
    "NotificationsResource",
    "AsyncNotificationsResource",
    "NotificationsResourceWithRawResponse",
    "AsyncNotificationsResourceWithRawResponse",
    "NotificationsResourceWithStreamingResponse",
    "AsyncNotificationsResourceWithStreamingResponse",
    "APIResource",
    "AsyncAPIResource",
    "APIResourceWithRawResponse",
    "AsyncAPIResourceWithRawResponse",
    "APIResourceWithStreamingResponse",
    "AsyncAPIResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["EmbedderConfigParam"]


class EmbedderConfigParam(TypedDict, total=False):
    api_type: Annotated[Literal["local", "remote"], PropertyInfo(alias="API_TYPE")]
    """The inference type (local or remote)."""

    batch_size: Annotated[int, PropertyInfo(alias="BATCH_SIZE")]
    """Smaller batch size for better parallelization."""

    embed_prefix: Annotated[str, PropertyInfo(alias="EMBED_PREFIX")]
    """How to embed the sentence for retrieval."""

    max_concurrent_requests: Annotated[int, PropertyInfo(alias="MAX_CONCURRENT_REQUESTS")]
    """Adjust concurrency level as needed."""

    model_name: Annotated[str, PropertyInfo(alias="MODEL_NAME")]
    """The name of the embedder model."""

    query_prefix: Annotated[str, PropertyInfo(alias="QUERY_PREFIX")]
    """How to embed the sentence for query."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["WorkspaceCopyResponse", "Result"]


class Result(BaseModel):
    source_doc_ext_id: str

    success: bool

    error: Optional[str] = None

    new_doc_ext_id: Optional[str] = None


class WorkspaceCopyResponse(BaseModel):
    detail: str

    documents_copied: Optional[int] = None

    results: Optional[List[Result]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypeAlias

__all__ = ["ParserConfigParam"]

ParserConfigParam: TypeAlias = object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UserCheckSSOStatusResponse"]


class UserCheckSSOStatusResponse(BaseModel):
    """SSO status response - indicates user registration state.

    States:
    - new_user: No user exists, show "Set Master Password"
    - local_exists: Local user exists, show "Link SSO Account? Enter your master password"
    - sso_exists: SSO user exists, show "Enter Master Password"
    """

    email: str

    status: str

    family_name: Optional[str] = None

    given_name: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["DoctagGenerateResponse"]


class DoctagGenerateResponse(BaseModel):
    """Response for generate annotations endpoint (202 Accepted).

    Returns the accepted doc and tag IDs that will be processed.
    """

    doc_ext_ids: List[str]
    """Document IDs that will be processed"""

    tag_ext_ids: List[str]
    """Tag IDs that will be processed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["QueryLlmConfigParam"]


class QueryLlmConfigParam(TypedDict, total=False):
    api_type: Annotated[Literal["local", "remote"], PropertyInfo(alias="API_TYPE")]
    """The inference type (local or remote)."""

    max_char_size_to_answer: Annotated[int, PropertyInfo(alias="MAX_CHAR_SIZE_TO_ANSWER")]
    """Maximum character size to answer."""

    max_tokens: Annotated[int, PropertyInfo(alias="MAX_TOKENS")]
    """Maximum number of tokens allowed."""

    model_name: Annotated[str, PropertyInfo(alias="MODEL_NAME")]
    """The name of the non-reasoning model to be used."""

    system_instruction: Annotated[str, PropertyInfo(alias="SYSTEM_INSTRUCTION")]
    """The system instruction string."""

    temperature: Annotated[float, PropertyInfo(alias="TEMPERATURE")]
    """Temperature value for randomness."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["WorkspaceCopyParams"]


class WorkspaceCopyParams(TypedDict, total=False):
    items: Required[SequenceNotStr[str]]
    """List of document external IDs to copy (e.g., ['doc-a1b2c3d4', 'doc-e5f6g7h8'])"""

    target_workspace_ext_id: Required[str]

    target_workspace_key: Annotated[str, PropertyInfo(alias="target-workspace-key")]

    workspace_key: Annotated[str, PropertyInfo(alias="workspace-key")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .user_response import UserResponse

__all__ = ["UserLoginResponse"]


class UserLoginResponse(BaseModel):
    """Login response with access token, session key, and user info.

    Used by: /user/login
    """

    access_token: str

    session_key: str

    user: UserResponse
    """Standard user representation used across all endpoints.

    Used for: login response, workspace users, contacts (when registered).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arbi import Arbi, AsyncArbi
from arbi._utils import parse_date
from tests.utils import assert_matches_type
from arbi.types.api import (
    DocumentUpdateResponse,
    DocumentUploadResponse,
    DocumentRetrieveResponse,
    DocumentGetParsedResponse,
    DocumentUploadFromURLResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocument:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Arbi) -> None:
        document = client.api.document.retrieve(
            external_ids=["string"],
        )
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.retrieve(
            external_ids=["string"],
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.retrieve(
            external_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.retrieve(
            external_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Arbi) -> None:
        document = client.api.document.update(
            documents=[{"external_id": "doc-bFXA5r3A"}],
        )
        assert_matches_type(DocumentUpdateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.update(
            documents=[
                {
                    "external_id": "doc-bFXA5r3A",
                    "doc_metadata": {
                        "doc_author": "doc_author",
                        "doc_date": parse_date("2019-12-27"),
                        "doc_nature": "doc_nature",
                        "doc_subject": "doc_subject",
                        "title": "title",
                    },
                    "shared": True,
                }
            ],
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentUpdateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.update(
            documents=[{"external_id": "doc-bFXA5r3A"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentUpdateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.update(
            documents=[{"external_id": "doc-bFXA5r3A"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentUpdateResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Arbi) -> None:
        document = client.api.document.delete(
            external_ids=["string"],
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.delete(
            external_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.delete(
            external_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_download(self, client: Arbi) -> None:
        document = client.api.document.download(
            document_ext_id="doc",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_download_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.download(
            document_ext_id="doc",
            workspace_key="workspace-key",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_download(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.download(
            document_ext_id="doc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_download(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.download(
            document_ext_id="doc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(object, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_download(self, client: Arbi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_ext_id` but received ''"):
            client.api.document.with_raw_response.download(
                document_ext_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_parsed(self, client: Arbi) -> None:
        document = client.api.document.get_parsed(
            stage="marker",
            document_ext_id="doc",
        )
        assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_parsed_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.get_parsed(
            stage="marker",
            document_ext_id="doc",
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_parsed(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.get_parsed(
            stage="marker",
            document_ext_id="doc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_parsed(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.get_parsed(
            stage="marker",
            document_ext_id="doc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_parsed(self, client: Arbi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_ext_id` but received ''"):
            client.api.document.with_raw_response.get_parsed(
                stage="marker",
                document_ext_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload(self, client: Arbi) -> None:
        document = client.api.document.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
        )
        assert_matches_type(DocumentUploadResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
            config_ext_id="config_ext_id",
            shared=True,
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentUploadResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_upload(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentUploadResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_upload(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentUploadResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_from_url(self, client: Arbi) -> None:
        document = client.api.document.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
        )
        assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_upload_from_url_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
            config_ext_id="config_ext_id",
            shared=True,
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_upload_from_url(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_upload_from_url(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_view(self, client: Arbi) -> None:
        document = client.api.document.view(
            document_ext_id="doc",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_view_with_all_params(self, client: Arbi) -> None:
        document = client.api.document.view(
            document_ext_id="doc",
            workspace_key="workspace-key",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_view(self, client: Arbi) -> None:
        response = client.api.document.with_raw_response.view(
            document_ext_id="doc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_view(self, client: Arbi) -> None:
        with client.api.document.with_streaming_response.view(
            document_ext_id="doc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(object, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_view(self, client: Arbi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_ext_id` but received ''"):
            client.api.document.with_raw_response.view(
                document_ext_id="",
            )


class TestAsyncDocument:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.retrieve(
            external_ids=["string"],
        )
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.retrieve(
            external_ids=["string"],
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.retrieve(
            external_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.retrieve(
            external_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.update(
            documents=[{"external_id": "doc-bFXA5r3A"}],
        )
        assert_matches_type(DocumentUpdateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.update(
            documents=[
                {
                    "external_id": "doc-bFXA5r3A",
                    "doc_metadata": {
                        "doc_author": "doc_author",
                        "doc_date": parse_date("2019-12-27"),
                        "doc_nature": "doc_nature",
                        "doc_subject": "doc_subject",
                        "title": "title",
                    },
                    "shared": True,
                }
            ],
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentUpdateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.update(
            documents=[{"external_id": "doc-bFXA5r3A"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentUpdateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.update(
            documents=[{"external_id": "doc-bFXA5r3A"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentUpdateResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.delete(
            external_ids=["string"],
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.delete(
            external_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.delete(
            external_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_download(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.download(
            document_ext_id="doc",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_download_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.download(
            document_ext_id="doc",
            workspace_key="workspace-key",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_download(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.download(
            document_ext_id="doc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_download(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.download(
            document_ext_id="doc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(object, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_download(self, async_client: AsyncArbi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_ext_id` but received ''"):
            await async_client.api.document.with_raw_response.download(
                document_ext_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_parsed(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.get_parsed(
            stage="marker",
            document_ext_id="doc",
        )
        assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_parsed_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.get_parsed(
            stage="marker",
            document_ext_id="doc",
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_parsed(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.get_parsed(
            stage="marker",
            document_ext_id="doc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_parsed(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.get_parsed(
            stage="marker",
            document_ext_id="doc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentGetParsedResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_parsed(self, async_client: AsyncArbi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_ext_id` but received ''"):
            await async_client.api.document.with_raw_response.get_parsed(
                stage="marker",
                document_ext_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
        )
        assert_matches_type(DocumentUploadResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
            config_ext_id="config_ext_id",
            shared=True,
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentUploadResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_upload(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentUploadResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_upload(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.upload(
            workspace_ext_id="wrk",
            files=[b"raw file contents"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentUploadResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_from_url(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
        )
        assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_upload_from_url_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
            config_ext_id="config_ext_id",
            shared=True,
            workspace_key="workspace-key",
        )
        assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_upload_from_url(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_upload_from_url(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.upload_from_url(
            urls=["string"],
            workspace_ext_id="wrk",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentUploadFromURLResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_view(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.view(
            document_ext_id="doc",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_view_with_all_params(self, async_client: AsyncArbi) -> None:
        document = await async_client.api.document.view(
            document_ext_id="doc",
            workspace_key="workspace-key",
        )
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_view(self, async_client: AsyncArbi) -> None:
        response = await async_client.api.document.with_raw_response.view(
            document_ext_id="doc",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(object, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_view(self, async_client: AsyncArbi) -> None:
        async with async_client.api.document.with_streaming_response.view(
            document_ext_id="doc",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(object, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_view(self, async_client: AsyncArbi) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `document_ext_id` but received ''"):
            await async_client.api.document.with_raw_response.view(
                document_ext_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .user_response import UserResponse

__all__ = ["WorkspaceGetUsersResponse"]

WorkspaceGetUsersResponse: TypeAlias = List[UserResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.api.user import setting_update_params
from ....types.api.user.setting_retrieve_response import SettingRetrieveResponse

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingRetrieveResponse:
        """Get current user's settings."""
        return self._get(
            "/api/user/settings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingRetrieveResponse,
        )

    def update(
        self,
        *,
        ai_mode: Optional[str] | Omit = omit,
        hide_online_status: Optional[bool] | Omit = omit,
        muted_users: Optional[SequenceNotStr[str]] | Omit = omit,
        pinned_workspaces: Optional[SequenceNotStr[str]] | Omit = omit,
        show_document_navigator: Optional[bool] | Omit = omit,
        show_help_page: Optional[bool] | Omit = omit,
        show_invite_tab: Optional[bool] | Omit = omit,
        show_security_settings: Optional[bool] | Omit = omit,
        show_smart_search: Optional[bool] | Omit = omit,
        show_templates: Optional[bool] | Omit = omit,
        show_thread_visualization: Optional[bool] | Omit = omit,
        subscription: Optional[setting_update_params.Subscription] | Omit = omit,
        tableviews: Optional[Iterable[setting_update_params.Tableview]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update user's settings (merge with existing).

        Note: subscription fields are managed automatically and cannot be patched.

        Args:
          subscription: Trial update - only trial_expires can be set, and only if currently null.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            "/api/user/settings",
            body=maybe_transform(
                {
                    "ai_mode": ai_mode,
                    "hide_online_status": hide_online_status,
                    "muted_users": muted_users,
                    "pinned_workspaces": pinned_workspaces,
                    "show_document_navigator": show_document_navigator,
                    "show_help_page": show_help_page,
                    "show_invite_tab": show_invite_tab,
                    "show_security_settings": show_security_settings,
                    "show_smart_search": show_smart_search,
                    "show_templates": show_templates,
                    "show_thread_visualization": show_thread_visualization,
                    "subscription": subscription,
                    "tableviews": tableviews,
                },
                setting_update_params.SettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/arbitrationcity/arbi-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingRetrieveResponse:
        """Get current user's settings."""
        return await self._get(
            "/api/user/settings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingRetrieveResponse,
        )

    async def update(
        self,
        *,
        ai_mode: Optional[str] | Omit = omit,
        hide_online_status: Optional[bool] | Omit = omit,
        muted_users: Optional[SequenceNotStr[str]] | Omit = omit,
        pinned_workspaces: Optional[SequenceNotStr[str]] | Omit = omit,
        show_document_navigator: Optional[bool] | Omit = omit,
        show_help_page: Optional[bool] | Omit = omit,
        show_invite_tab: Optional[bool] | Omit = omit,
        show_security_settings: Optional[bool] | Omit = omit,
        show_smart_search: Optional[bool] | Omit = omit,
        show_templates: Optional[bool] | Omit = omit,
        show_thread_visualization: Optional[bool] | Omit = omit,
        subscription: Optional[setting_update_params.Subscription] | Omit = omit,
        tableviews: Optional[Iterable[setting_update_params.Tableview]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update user's settings (merge with existing).

        Note: subscription fields are managed automatically and cannot be patched.

        Args:
          subscription: Trial update - only trial_expires can be set, and only if currently null.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            "/api/user/settings",
            body=await async_maybe_transform(
                {
                    "ai_mode": ai_mode,
                    "hide_online_status": hide_online_status,
                    "muted_users": muted_users,
                    "pinned_workspaces": pinned_workspaces,
                    "show_document_navigator": show_document_navigator,
                    "show_help_page": show_help_page,
                    "show_invite_tab": show_invite_tab,
                    "show_security_settings": show_security_settings,
                    "show_smart_search": show_smart_search,
                    "show_templates": show_templates,
                    "show_thread_visualization": show_thread_visualization,
                    "subscription": subscription,
                    "tableviews": tableviews,
                },
                setting_update_params.SettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.retrieve = to_raw_response_wrapper(
            settings.retrieve,
        )
        self.update = to_raw_response_wrapper(
            settings.update,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.retrieve = async_to_raw_response_wrapper(
            settings.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            settings.update,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.retrieve = to_streamed_response_wrapper(
            settings.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            settings.update,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.retrieve = async_to_streamed_response_wrapper(
            settings.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            settings.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .doc_response import DocResponse as DocResponse
from .user_response import UserResponse as UserResponse
from .embedder_config import EmbedderConfig as EmbedderConfig
from .reranker_config import RerankerConfig as RerankerConfig
from .query_llm_config import QueryLlmConfig as QueryLlmConfig
from .retriever_config import RetrieverConfig as RetrieverConfig
from .title_llm_config import TitleLlmConfig as TitleLlmConfig
from .tag_create_params import TagCreateParams as TagCreateParams
from .tag_update_params import TagUpdateParams as TagUpdateParams
from .user_login_params import UserLoginParams as UserLoginParams
from .workspace_response import WorkspaceResponse as WorkspaceResponse
from .parser_config_param import ParserConfigParam as ParserConfigParam
from .tag_create_response import TagCreateResponse as TagCreateResponse
from .tag_delete_response import TagDeleteResponse as TagDeleteResponse
from .tag_update_response import TagUpdateResponse as TagUpdateResponse
from .user_login_response import UserLoginResponse as UserLoginResponse
from .chunker_config_param import ChunkerConfigParam as ChunkerConfigParam
from .config_create_params import ConfigCreateParams as ConfigCreateParams
from .user_logout_response import UserLogoutResponse as UserLogoutResponse
from .user_register_params import UserRegisterParams as UserRegisterParams
from .embedder_config_param import EmbedderConfigParam as EmbedderConfigParam
from .model_citation_config import ModelCitationConfig as ModelCitationConfig
from .reranker_config_param import RerankerConfigParam as RerankerConfigParam
from .workspace_copy_params import WorkspaceCopyParams as WorkspaceCopyParams
from .assistant_query_params import AssistantQueryParams as AssistantQueryParams
from .config_create_response import ConfigCreateResponse as ConfigCreateResponse
from .config_delete_response import ConfigDeleteResponse as ConfigDeleteResponse
from .document_delete_params import DocumentDeleteParams as DocumentDeleteParams
from .document_update_params import DocumentUpdateParams as DocumentUpdateParams
from .document_upload_params import DocumentUploadParams as DocumentUploadParams
from .query_llm_config_param import QueryLlmConfigParam as QueryLlmConfigParam
from .retriever_config_param import RetrieverConfigParam as RetrieverConfigParam
from .title_llm_config_param import TitleLlmConfigParam as TitleLlmConfigParam
from .workspace_share_params import WorkspaceShareParams as WorkspaceShareParams
from .workspace_copy_response import WorkspaceCopyResponse as WorkspaceCopyResponse
from .workspace_update_params import WorkspaceUpdateParams as WorkspaceUpdateParams
from .config_retrieve_response import ConfigRetrieveResponse as ConfigRetrieveResponse
from .document_retrieve_params import DocumentRetrieveParams as DocumentRetrieveParams
from .document_update_response import DocumentUpdateResponse as DocumentUpdateResponse
from .document_upload_response import DocumentUploadResponse as DocumentUploadResponse
from .user_verify_email_params import UserVerifyEmailParams as UserVerifyEmailParams
from .workspace_share_response import WorkspaceShareResponse as WorkspaceShareResponse
from .assistant_retrieve_params import AssistantRetrieveParams as AssistantRetrieveParams
from .workspace_delete_response import WorkspaceDeleteResponse as WorkspaceDeleteResponse
from .document_retrieve_response import DocumentRetrieveResponse as DocumentRetrieveResponse
from .health_get_models_response import HealthGetModelsResponse as HealthGetModelsResponse
from .notification_create_params import NotificationCreateParams as NotificationCreateParams
from .notification_delete_params import NotificationDeleteParams as NotificationDeleteParams
from .notification_list_response import NotificationListResponse as NotificationListResponse
from .notification_update_params import NotificationUpdateParams as NotificationUpdateParams
from .user_verify_email_response import UserVerifyEmailResponse as UserVerifyEmailResponse
from .conversation_share_response import ConversationShareResponse as ConversationShareResponse
from .model_citation_config_param import ModelCitationConfigParam as ModelCitationConfigParam
from .user_change_password_params import UserChangePasswordParams as UserChangePasswordParams
from .user_list_products_response import UserListProductsResponse as UserListProductsResponse
from .workspace_get_tags_response import WorkspaceGetTagsResponse as WorkspaceGetTagsResponse
from .config_get_versions_response import ConfigGetVersionsResponse as ConfigGetVersionsResponse
from .conversation_delete_response import ConversationDeleteResponse as ConversationDeleteResponse
from .document_get_parsed_response import DocumentGetParsedResponse as DocumentGetParsedResponse
from .health_check_models_response import HealthCheckModelsResponse as HealthCheckModelsResponse
from .notification_create_response import NotificationCreateResponse as NotificationCreateResponse
from .notification_update_response import NotificationUpdateResponse as NotificationUpdateResponse
from .user_check_sso_status_params import UserCheckSSOStatusParams as UserCheckSSOStatusParams
from .workspace_get_stats_response import WorkspaceGetStatsResponse as WorkspaceGetStatsResponse
from .workspace_get_users_response import WorkspaceGetUsersResponse as WorkspaceGetUsersResponse
from .workspace_remove_user_params import WorkspaceRemoveUserParams as WorkspaceRemoveUserParams
from .user_change_password_response import UserChangePasswordResponse as UserChangePasswordResponse
from .user_list_workspaces_response import UserListWorkspacesResponse as UserListWorkspacesResponse
from .user_check_sso_status_response import UserCheckSSOStatusResponse as UserCheckSSOStatusResponse
from .workspace_remove_user_response import WorkspaceRemoveUserResponse as WorkspaceRemoveUserResponse
from .document_upload_from_url_params import DocumentUploadFromURLParams as DocumentUploadFromURLParams
from .health_retrieve_status_response import HealthRetrieveStatusResponse as HealthRetrieveStatusResponse
from .conversation_update_title_params import ConversationUpdateTitleParams as ConversationUpdateTitleParams
from .workspace_get_documents_response import WorkspaceGetDocumentsResponse as WorkspaceGetDocumentsResponse
from .document_upload_from_url_response import DocumentUploadFromURLResponse as DocumentUploadFromURLResponse
from .notification_get_schemas_response import NotificationGetSchemasResponse as NotificationGetSchemasResponse
from .workspace_create_protected_params import WorkspaceCreateProtectedParams as WorkspaceCreateProtectedParams
from .conversation_update_title_response import ConversationUpdateTitleResponse as ConversationUpdateTitleResponse
from .document_date_extractor_llm_config import DocumentDateExtractorLlmConfig as DocumentDateExtractorLlmConfig
from .conversation_delete_message_response import ConversationDeleteMessageResponse as ConversationDeleteMessageResponse
from .workspace_get_conversations_response import WorkspaceGetConversationsResponse as WorkspaceGetConversationsResponse
from .conversation_retrieve_message_response import (
    ConversationRetrieveMessageResponse as ConversationRetrieveMessageResponse,
)
from .conversation_retrieve_threads_response import (
    ConversationRetrieveThreadsResponse as ConversationRetrieveThreadsResponse,
)
from .document_date_extractor_llm_config_param import (
    DocumentDateExtractorLlmConfigParam as DocumentDateExtractorLlmConfigParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["SubscriptionCreateResponse"]


class SubscriptionCreateResponse(BaseModel):
    """Response containing Stripe checkout session client secret."""

    client_secret: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ..._models import BaseModel

__all__ = ["HealthRetrieveStatusResponse", "ModelsHealth", "ModelsHealthModel", "Service"]


class ModelsHealthModel(BaseModel):
    model: str

    status: str

    detail: Optional[str] = None


class ModelsHealth(BaseModel):
    application: str

    models: List[ModelsHealthModel]


class Service(BaseModel):
    name: str

    status: str

    detail: Optional[str] = None

    service_info: Optional[Dict[str, object]] = None


class HealthRetrieveStatusResponse(BaseModel):
    """
    Consolidated health response containing all system status and version information
    """

    status: str

    available_models: Optional[List[str]] = None

    backend_git_hash: Optional[str] = None

    frontend_docker_version: Optional[str] = None

    models_health: Optional[ModelsHealth] = None

    services: Optional[List[Service]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["NotificationUpdateParams", "Update"]


class NotificationUpdateParams(TypedDict, total=False):
    updates: Required[Iterable[Update]]


class Update(TypedDict, total=False):
    """Single notification update for bulk PATCH.

    Supports two operations:
    - content: Re-encrypt content (key rotation)
    - read: Mark as read (only recipient can do this)
    """

    external_id: Required[str]

    content: Optional[str]

    read: Optional[bool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ConfigGetVersionsResponse", "Version"]


class Version(BaseModel):
    created_at: str

    external_id: str

    title: Optional[str] = None


class ConfigGetVersionsResponse(BaseModel):
    versions: List[Version]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["DoctagDeleteParams"]


class DoctagDeleteParams(TypedDict, total=False):
    doc_ext_ids: Required[SequenceNotStr[str]]

    tag_ext_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["NotificationDeleteParams"]


class NotificationDeleteParams(TypedDict, total=False):
    external_ids: Required[SequenceNotStr[str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DocumentDateExtractorLlmConfig"]


class DocumentDateExtractorLlmConfig(BaseModel):
    api_type: Optional[Literal["local", "remote"]] = FieldInfo(alias="API_TYPE", default=None)
    """The inference type (local or remote)."""

    max_char_context_to_answer: Optional[int] = FieldInfo(alias="MAX_CHAR_CONTEXT_TO_ANSWER", default=None)
    """Maximum characters in document for context."""

    max_tokens: Optional[int] = FieldInfo(alias="MAX_TOKENS", default=None)
    """Maximum number of tokens allowed."""

    api_model_name: Optional[str] = FieldInfo(alias="MODEL_NAME", default=None)
    """The name of the non-reasoning model to be used."""

    system_instruction: Optional[str] = FieldInfo(alias="SYSTEM_INSTRUCTION", default=None)

    temperature: Optional[float] = FieldInfo(alias="TEMPERATURE", default=None)
    """Temperature value for randomness."""
